/**
 * <p>File: Exm020203.java</p>
 * <p>Title: Numerical methods</p>
 * <p>Description: Course on numerical analysis</p>
 * <p>Date: December 2011</p>
 * @author Faton M. Berisha
 * @version 6.0.0
 */

import numeric.*;

public class Exm020203 {
	public static void main(String[] args) {
		EquationSolver solverA = new EquationSolver();
		solverA.setTolerance(1E-9);
		solverA.setNumIterations(30);

		EquationSolver solverB = new EquationSolver();
		solverB.setTolerance(1E-9);
		solverB.setNumIterations(30);

		EquationSolver solverC = new EquationSolver();
		solverC.setTolerance(1E-9);
		solverC.setNumIterations(30);

		EquationSolver solverD = new EquationSolver();
		solverD.setTolerance(1E-9);
		solverD.setNumIterations(30);

		EquationSolver solverE = new EquationSolver();
		solverE.setTolerance(1E-9);
		solverE.setNumIterations(30);

		Function gA = new IterativeFunctionA();
		Function gB = new IterativeFunctionB();
		Function gC = new IterativeFunctionC();
		Function gD = new IterativeFunctionD();
		Differentiable f = new DifferentiableE();

		new EquationDemoView(solverA);
		new EquationDemoView(solverB);
		new EquationDemoView(solverC);
		new EquationDemoView(solverD);
		new EquationDemoView(solverE);

		EquationDemoView.setNumSignificantDigits(10);

		try {
			solverA.fixedPoint(gA, 1.5);
		} catch (ArithmeticException e) {
			System.err.println(e.getMessage());
		}

		try {
			solverB.fixedPoint(gB, 1.5);
		} catch (ArithmeticException e) {
			System.err.println(e.getMessage());
		}

		try {
			solverC.fixedPoint(gC, 1.5);
		} catch (ArithmeticException e) {
			System.err.println(e.getMessage());
		}

		try {
			solverD.fixedPoint(gD, 1.5);
		} catch (ArithmeticException e) {
			System.err.println(e.getMessage());
		}

		try {
			solverE.newton(f, 1.5);
		} catch (ArithmeticException e) {
			System.err.println(e.getMessage());
		}
	}

	private static class IterativeFunctionA implements Function {
		public double value(double x) {
			return x - x * x * x - 4 * x * x + 10;
		}
	}

	private static class IterativeFunctionB implements Function {
		public double value(double x) {
			return Math.sqrt(10 / x - 4 * x);
		}
	}

	private static class IterativeFunctionC implements Function {
		public double value(double x) {
			return Math.sqrt(10 - x * x * x) / 2;
		}
	}

	private static class IterativeFunctionD implements Function {
		public double value(double x) {
			return Math.sqrt(10 / (4 + x));
		}
	}

	private static class DifferentiableE implements Differentiable {
		public double value(double x) {
			return x * x * x + 4 * x * x - 10;
		}

		public double derivative(double x) {
			return 3 * x * x + 8 * x;
		}
	}
}
