import numeric.*;

public class Exm100201 {
	public static void main(String[] args) {
		double[] initial = { 0.1, 0.1, -0.1 };
		SystemSolver solver = new SystemSolver(initial.length);

		solver.setTolerance(1E-8);
		solver.setNumIterations(20);
		SystemDemoView.setNumSignificantDigits(8);

		PartiallyDifferentiable f = new VectorFunctionF();
		new SystemDemoView("Metoda e Newton-it", solver);
		try {
			solver.newton(f, initial);
		} catch (ArithmeticException e) {
			System.err.println(e.getMessage());
		}
	}

	private static class VectorFunctionF implements
			PartiallyDifferentiable {
		public double value(int i, double[] x) {
			double answer = 0;
			switch (i) {
			case 0: {
				answer = 3 * x[0] - Math.cos(x[1] * x[2]) - 1.0/2.0;
				break;
			}
			case 1: {
				answer = x[0] * x[0] - 81 * (x[1] + 0.1) * (x[1] + 0.1)
						+ Math.sin(x[2]) + 1.06;
				break;
			}
			case 2: {
				answer = Math.exp(- x[0] * x[1]) + 20 * x[2]
				        + (10 * Math.PI - 3)/3.0;
				break;
			}
			}
			return answer;
		}

		public double partialDerivative(int i, int j, double[] x) {
			double answer = 0;
			switch (i) {
			case 0:
				switch (j) {
				case 0:
					answer = 3;
					break;
				case 1:
					answer = x[2] * Math.sin(x[1] * x[2]);
					break;
				case 2:
					answer = x[1] * Math.sin(x[1] * x[2]);
					break;
				}
				break;
			case 1:
				switch (j) {
				case 0:
					answer = 2 * x[0];
					break;
				case 1:
					answer = -162 * (x[1] + 0.1);
					break;
				case 2:
					answer = Math.cos(x[2]);
					break;
				}
				break;
			case 2:
				switch (j) {
				case 0:
					answer = -x[1] * Math.exp(-x[0] * x[1]);
					break;
				case 1:
					answer = -x[0] * Math.exp(-x[0] * x[1]);
					break;
				case 2:
					answer = 20;
					break;
				}
				break;
			}
			return answer;
		}
	}
}
